@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2008-2012 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -17,12 +17,15 @@
 from random import choice
 import os
 import re
+import math
 from os import path
 from calculate.lib import cl_overriding
 import sys
 import getpass
 from types import StringType
 import string
+import pwd
+import grp
 
 from calculate.lib.cl_lang import setLocalTranslate
 setLocalTranslate('cl_lib3',sys.modules[__name__])
@@ -411,6 +414,16 @@ def getGroups(prefix="/"):
            map(lambda x:x.split(':')[0],
            readLinesFile(path.join(prefix,'etc/group'))))
 
+def getPagesInterval(count,offset,length):
+    """
+    Получить интервал страниц (например 2 из 5, при (5,5,15)
+    """
+    rt = max(0,length-offset-count)
+    lt = min(length,offset)
+    lt = int(math.ceil(float(lt)/count))
+    rt = int(math.ceil(float(rt)/count))
+    return (lt + 1, rt + lt + 1)
+
 def mountEcryptfs(userName,userPwd,userDir):
     """
     Подключить ecryptfs
@@ -439,7 +452,7 @@ def mountEcryptfs(userName,userPwd,userDir):
         else:
             raise Exception
     except:
-        raise CommonError("Failed to unwrap passphrase")
+        raise CommonError(_("Failed to unwrap the passphrase"))
 
     # добавить passphrase в ключи ядра
     p = process('/usr/bin/ecryptfs-add-passphrase', '--fnek', '-',stderr=STDOUT)
@@ -452,7 +465,7 @@ def mountEcryptfs(userName,userPwd,userDir):
         ecryptfs_sig, ecryptfs_fnek_sig = \
             readLinesFile(path.join(ecryptUserName,".ecryptfs/Private.sig"))
     except ValueError:
-        raise CommonError("Failed to parse Private.sig")
+        raise CommonError(_("Failed to parse Private.sig"))
 
     # подключить шифрованный раздел
     mountProcess = process('/sbin/mount.ecryptfs',
@@ -471,3 +484,23 @@ def mountEcryptfs(userName,userPwd,userDir):
     # отправить пароль через stdin
     mountProcess.write("passphrase_passwd="+userPwd)
     return mountProcess.success()
+
+def isBootstrapDataOnly(directory):
+    """
+    Каталог содержит только сертификат, созданный командой cl-core
+    """
+    from calculate.lib.utils.files import (process,readLinesFile,STDOUT,
+                                           isMount)
+    userCalculate = path.join(directory,".calculate")
+    return (set(listDirectory(directory)) == set([".calculate"]) and
+            set(listDirectory(userCalculate)) == set(["client_cert"]))
+
+def getPortageUidGid():
+    """
+    Получить uid и gid пользователя portage
+    """
+    data = pwd.getpwnam('portage')
+    if data:
+        return data.pw_uid,data.pw_gid
+    else:
+        return 250,250
