@@ -1,6 +1,6 @@
 #-*- coding: utf-8 -*-
 
-# Copyright 2008-2012 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -19,13 +19,30 @@ from os import path
 import subprocess
 from subprocess import Popen,PIPE,STDOUT
 import stat
-from shutil import copytree, rmtree
+from shutil import copytree, rmtree,copy2
 from calculate.lib import cl_overriding
 import re
 import sys
 from itertools import *
 import tarfile
 
+from contextlib import contextmanager
+import signal
+
+@contextmanager
+def timeout(seconds):
+    def timeout_handler(signum,frame):
+        pass
+
+    original_handler = signal.signal(signal.SIGALRM, timeout_handler)
+
+    try:
+        signal.alarm(seconds)
+        yield
+    finally:
+        signal.alarm(0)
+        signal.signal(signal.SIGALRM, original_handler)
+
 try:
     from magic import open as type_file, MAGIC_NONE as MAGIC_NONE
 except ImportError:
@@ -38,6 +55,13 @@ except ImportError:
 from calculate.lib.cl_lang import setLocalTranslate
 setLocalTranslate('cl_lib3',sys.modules[__name__])
 
+reSearch = lambda pattern,listar:map(lambda x:x.groups(),
+                        filter(None,
+                        map(pattern.search,listar)))
+
+class FilesError(Exception):
+    pass
+
 class process:
     """Execute system command by Popen
     
@@ -84,7 +108,11 @@ class process:
 
         self.stderr = kwarg.get("stderr",PIPE)
         self.cwd = kwarg.get("cwd",None)
-        self.command = [command] + list(params)
+        self.command = getProgPath(command)
+        if not self.command:
+            raise FilesError(_("Program %s is not found")%
+                             command)
+        self.command = [self.command] + list(params)
         self.stdin = stdin
         self.iter = None
         self.pipe = None
@@ -403,7 +431,10 @@ class processProgress(process):
             self.pipe.kill()
             raise
         finally:
-            self.cacheresult = "\n".join(self.cacheresult)
+            if self.cacheresult:
+                self.cacheresult = "\n".join(self.cacheresult)
+            else:
+                self.cacheresult = ""
 
 def countFiles(dirpath,onefilesystem=True):
     """
@@ -609,11 +640,19 @@ def readLinesFile(filename):
     finally:
         raise StopIteration
 
-def readFile(filename):
-    """Read whole file or return empty string"""
+def readFile(filename,tailbyte=None):
+    """
+    Прочитать целый файл или вернуть пустую строку.
+    tailbyte: прочитать только последнее указанное количество байт
+    """
     try:
         if path.exists(filename):
-            return open(filename,'r').read()
+            with open(filename,'r') as f:
+                if tailbyte:
+                    seeksize = max(0,os.stat(filename).st_size-tailbyte)
+                    if seeksize:
+                        f.seek(seeksize)
+                return f.read()
     except (OSError,IOError):
         pass
     return ""
@@ -649,7 +688,7 @@ def checkUtils(*utils):
     for util in utils:
         utilPath = getProgPath(util)
         if not utilPath:
-            raise Exception(_("Command not found '%s'")%
+            raise FilesError(_("Command not found '%s'")%
                                      path.basename(util))
         retval.append(utilPath)
     if len(retval) == 1:
@@ -774,39 +813,27 @@ class FStab(object):
         return bool(filter(filterfunc,self.cache))
 
 def tarLinks(rootpath,archpath,skip=[]):
-    """Add symbolic links to archive file"""
+    """
+    Поместить все симлинки в архив
+    """
     links = []
-    if skip:
-        reSkip = re.compile("|".join(map(lambda x:x.replace("*",".*"),
-            skip))).search
-    else:
-        reSkip = lambda x:False
     lenprefix = len(path.normpath(rootpath))+1
     if path.exists(archpath):
         os.unlink(archpath)
     # create arch
     tar = tarfile.open(archpath,"w:bz2")
     # find links
-    for root, dirs, files in os.walk(rootpath):
-        for link in  filter(os.path.islink,
-                     map(lambda x:path.join(root,x),
-                     dirs+files)):
-            # try add link
-            try:
-                if not reSkip(link):
-                    ti = tar.gettarinfo(link)
-                    ti.name = link[lenprefix:]
-                    tar.addfile(ti)
-                    links.append(link)
-            except OSError:
-                pass
-        # discard mounted paths
-        removeDirs = map(lambda x:x[0],
-                     filter(lambda x:path.islink(x[1]) or path.ismount(x[1]),
-                     map(lambda x:(x,path.join(root,x)),
-                     dirs)))
-        map(lambda x:dirs.remove(x),
-        removeDirs)
+    if skip:
+        reSkip = re.compile("|".join(map(lambda x:x.replace("*",".*"),
+            skip))).search
+    else:
+        reSkip = lambda x:False
+    for link in ifilterfalse(reSkip,
+                find(rootpath,filetype="l",onefilesystem=True)):
+        ti = tar.gettarinfo(link)
+        ti.name = link[lenprefix:]
+        tar.addfile(ti)
+        links.append(link)
     tar.close()
     return links
 
@@ -824,3 +851,110 @@ def getch():
     finally:
         termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
     return ch
+
+def find(directory,onefilesystem=False,filetype=None,
+         fullpath=True, depth=None, downfilter=None):
+    """
+    Поиск файлов в каталоге
+    """
+    ldirectory = len(directory)
+    if downfilter or onefilesystem or depth:
+        _downfilter = lambda dp: (
+            (depth is None or dp[ldirectory:].count('/') < depth) and
+            (not downfilter or downfiler(dp)) and
+            (not onefilesystem or not path.ismount(dp)))
+    else:
+        _downfilter = None
+
+    if filetype:
+        def dirfilter(dp):
+            link = path.islink(dp)
+            return ("l" in filetype and link) or ("d" in filetype and not link)
+        _dirfilter = dirfilter
+    else:
+        _dirfilter = None
+
+    if fullpath or _dirfilter or _downfilter:
+        _dirfullpath = lambda root,fn: path.join(root,fn)
+    else:
+        _dirfullpath = lambda root,fn:fn
+
+    if filetype:
+        def filefilter(fp):
+            link = path.islink(fp)
+            return ("l" in filetype and link) or ("f" in filetype and not link)
+        _filefilter = filefilter
+    else:
+        _filefilter = None
+
+    if fullpath or _filefilter:
+        _filefullpath = lambda root,fn: path.join(root,fn)
+    else:
+        _filefullpath = lambda root,fn:fn
+
+    for root,directories,files in os.walk(directory):
+        for fn in files:
+            fp = _filefullpath(root,fn)
+            if not _filefilter or _filefilter(fp):
+                yield fp if fullpath else fn
+        for dn in directories[:]:
+            dp = _dirfullpath(root,dn)
+            if not _dirfilter or _dirfilter(dp):
+                yield dp if fullpath else dn
+            if _downfilter and not _downfilter(dp):
+                directories.remove(dn)
+
+def copyWithPath(fn,dest,prefix=None):
+    """
+    Копировать файл в dst с созданием каталогов
+
+    source: префикс исходного файла
+    dest: каталог для копирывания
+    fn: имя файла
+    """
+    try:
+        if prefix:
+            destFile = pathJoin(dest,fn[len(prefix):])
+        else:
+            destFile = pathJoin(dest,path.basename(fn))
+        destDir = path.dirname(destFile)
+        if not path.exists(destDir):
+            os.makedirs(destDir)
+        copy2(fn,destFile)
+    except Exception as e:
+        raise FilesError(
+            _("Failed to copy '%(src)s' to '%(dst)s'")%
+            {'src':fn,
+             'dst':destFile})
+
+def getLoopFromPath(directory):
+    """
+    Получить loop, использующие файлы в данной директории
+    """
+    losetup = getProgPath('losetup')
+    p = process(losetup,'-a')
+    rePattern = re.compile('(^\/dev\/loop[^:]+)[^(]+\(([^)]+)\).*')
+    return map(lambda x:x[0],
+            filter(lambda x:x[1].startswith(directory),
+            reSearch(rePattern,
+            p.readlines())))
+
+def getMdRaidDevices():
+    """
+    Получить словарь: какой mdraid какие использует устройства
+    """
+    reMdInfo = re.compile('^(\w+)\s+:\s+active.*?raid\d+\s+(.*)')
+    return dict(map(lambda x:(x[0],map(lambda x:x.partition('[')[0],
+                            x[1].split())),
+           reSearch(reMdInfo,readLinesFile('/proc/mdstat'))))
+
+def sambaPasswordCheck(username,password,server,resource):
+    """
+    Подключиться к указанному samba ресурсу сервера по логину паролю
+    """
+    smbclient = getProgPath('/usr/sbin/smbclient')
+    if not smbclient:
+        raise FilesError(_("Command smbclient is not found"))
+    p = process(smbclient,"-U",username,"//%s/%s"%(server,resource),
+                envdict={'PASSWD':password})
+    return p.success()
